@@ -1,6 +1,6 @@
 // This may look like C code, but it's really -*- C++ -*-
 /*
- * Copyright (C) 2008 Emweb bvba, Kessel-Lo, Belgium.
+ * Copyright (C) 2008 Emweb bv, Herent, Belgium.
  *
  * See the LICENSE file for terms of use.
  */
@@ -14,10 +14,16 @@
 #include <Wt/WSignal.h>
 #include <Wt/WJavaScript.h>
 
+#include <bitset>
+#include <cassert>
 #include <list>
 #include <vector>
 #include <sstream>
 
+#ifndef WT_TARGET_JAVA
+#include <type_traits>
+#endif // WT_TARGET_JAVA
+
 namespace Wt {
 
 enum class RepaintFlag {
@@ -44,20 +50,12 @@ class WCssTextRule;
  * widget and dialogs, have a parent which is usually a
  * WContainerWidget.
  *
- * \if cpp
- *
- * When a widget is deleted, it is also visually removed from the
- * user-interface and all children are deleted recursively.
- *
- * \endif
- *
  * This is an abstract base class. Implementations derive either from
  * the abstract WWebWidget (for basic widgets with a direct HTML
  * counter-part) or from the abstract WCompositeWidget (for anything
- * else). To add a %WWebWidget directly to a parent container, either
- * specify the parent in the constructor (which is conventionally the
- * last constructor argument), or add the widget to the parent using
- * WContainerWidget::addWidget(). Alternatively, you may add the widget
+ * else). To add a %WWidget to a parent container add the
+ * widget to the parent using WContainerWidget::addWidget().
+ * Alternatively, you may add the widget
  * to a layout manager set for a %WContainerWidget.
  *
  * A widget provides methods to manage its decorative style base on
@@ -89,6 +87,30 @@ public:
    */
   virtual std::unique_ptr<WWidget> removeWidget(WWidget *widget);
 
+#ifndef WT_TARGET_JAVA
+  /*! \brief Removes a child widget.
+   *
+   * This is an overload that automatically casts the returned value to a unique_ptr<Widget> for convenience
+   *
+   * This is implemented as:
+   * \code
+   * return std::unique_ptr<Widget>(static_cast<Widget*>(removeWidget(static_cast<WWidget*>(widget)).release()));
+   * \endcode
+   */
+  template<typename Widget>
+#ifdef DOXYGEN_ONLY
+  std::unique_ptr<Widget> removeWidget(Widget *widget)
+#else // DOXYGEN_ONLY
+  typename std::enable_if<std::is_convertible<Widget*,WWidget*>::value && !std::is_same<Widget,WWidget>::value, std::unique_ptr<Widget>>::type removeWidget(Widget *widget)
+#endif // DOXYGEN_ONLY
+  {
+    std::unique_ptr<WWidget> result = removeWidget(static_cast<WWidget*>(widget));
+    // The result is null or the given widget, guaranteeing that result.release() can be casted to Widget*
+    assert(result == nullptr || result.get() == widget);
+    return std::unique_ptr<Widget>(static_cast<Widget*>(result.release()));
+  }
+#endif // WT_TARGET_JAVA
+
   /*! \brief Removes the widget from its parent.
    *
    * This is equivalent to parent()->removeWidget(this);
@@ -129,7 +151,7 @@ public:
    * scheme. The widget may be shifted to the left or right by
    * specifying an offset for the \link Wt::Side::Left left\endlink or \link
    * Wt::Side::Right right\endlink) side. The widget may be shifted
-   * vertically, by specifying an offset for the \link Wt::AlignmentFlag::Top
+   * vertically, by specifying an offset for the \link Wt::Side::Top
    * top\endlink or \link Wt::Side::Bottom bottom\endlink side.
    *
    * For an \link Wt::PositionScheme::Absolute absolutely positioned\endlink widget,
@@ -370,8 +392,7 @@ public:
    *
    * This lets the widget float to one of the sides of the parent
    * widget, at the current line. A typical use is to position images
-   * within text. Valid values for Side or \if cpp Wt::None \elseif
-   * java {javadoclink Side#None None} \endif, Wt::Side::Left or Wt::Side::Right.
+   * within text. Valid values for Side Side::Left, or Side::Right.
    *
    * This applies to CSS-based layout.
    */
@@ -1353,10 +1374,10 @@ private:
   friend class WCompositeWidget;
   friend class WFileUpload;
   friend class WGLWidget;
+  friend class WLeafletMap;
   friend class WMenuItem;
   friend class WPaintedWidget;
   friend class WPopupWidget;
-  friend class WScrollArea;
   friend class WTemplate;
   friend class WViewWidget;
   friend class WWebWidget;
